@@ -3,12 +3,12 @@ from csv import writer
 from ConfigParser import ConfigParser
 from StringIO import StringIO
 import os, shutil
-from types import FileType, StringType, ListType, TupleType
+from types import FileType, ListType, TupleType, StringTypes
 from Globals import InitializeClass
 
 from AccessControl import ClassSecurityInfo, AuthEncoding
 from AccessControl.Permissions import use_mailhost_services
-from Acquisition import aq_base
+from Acquisition import aq_inner
 from OFS.SimpleItem import SimpleItem
 from OFS.Image import File
 from DateTime import DateTime
@@ -28,7 +28,7 @@ from Products.CMFCore.utils import UniqueObject
 from Products.CMFCore.utils import getToolByName
 
 import zLOG
-from config import FOLDERS, SEPARATOR, PROPS, ALLPROPS, NONATPROPS, TYPEMAP
+from config import SEPARATOR, PROPS, ALLPROPS, NONATPROPS, TYPEMAP
 
 
 class ContentExporter(UniqueObject, SimpleItem):
@@ -85,7 +85,7 @@ class ContentExporter(UniqueObject, SimpleItem):
         if root:
             if not portal:
                 portal = getToolByName(self, 'portal_url').getPortalObject()
-            self.export(root=root, portal=portal)
+            self.export(root=root, portal=portal, users=request.get('exportusers',None))
         if hasattr(request, 'RESPONSE'):
             request.RESPONSE.redirect('manage_export')
 
@@ -118,21 +118,41 @@ class ContentExporter(UniqueObject, SimpleItem):
             filepath=os.path.join(folderpath,filename)
             if os.path.exists(filepath) and os.path.isfile(filepath):
                 os.remove(filepath)
+            ofd = None
             try:
-                ofd = os.open(filepath,os.O_CREAT | os.O_WRONLY | os.O_APPEND)
-                try:
-                    if type(data) == type(''):
+                # Treat everything as binary if Windows so line endings dont get tampered with
+                if hasattr(os, 'O_BINARY'):
+                    ofd = os.open(filepath,os.O_CREAT | os.O_WRONLY | os.O_APPEND | os.O_BINARY)
+                else:
+                    ofd = os.open(filepath,os.O_CREAT | os.O_WRONLY | os.O_APPEND)
+            except:
+                self.out.append("Failed to open %s for writing" % filepath)
+            if ofd:
+                binary = 0
+                if type(data) in StringTypes:
+                    try:
                         os.write(ofd,data)
-                    else:
+                    except:
+                        binary = 1
+                else:
+                    binary = 1
+                if binary:
+                    # Try to cope with string buffers or strings or None
+                    try:
                         while data is not None:
                             os.write(ofd, data.data)
                             data = data.next
-                except:
-                    self.out.append("Sorry failed to write to %s due to %s" \
-                                  (filepath,str(os.error)))
+                    except:
+                        try:
+                            os.write(ofd, data.data)
+                        except:
+                            try:
+                                data = str(data.data)
+                                if data:
+                                    os.write(ofd, data)
+                            except Exception, error:
+                                self.out.append("Sorry failed to write to %s due to %s" % (filepath,error))
                 os.close(ofd)
-            except:
-                self.out.append("Failed to open %s for writing" % filepath)
         #debug self.out.append('Wrote %s' % filepath)
         return
 
@@ -194,9 +214,9 @@ class ContentExporter(UniqueObject, SimpleItem):
         self.write_file('/acl_users','source_groups.xml',
                                      template(info=info))
         output = ostream.getvalue()
-        # Windows compatibility fix?
-        if os.path.join('a','b')=='a\b':
-            output = output.replace('\r\r\n','\r\n')
+        # strip Windows line endings
+        if hasattr(os, 'O_BINARY'):
+            output = output.replace('\r','')
         self.write_file(path,'.objects',output)
         self.out.append('Exported member data')
 
@@ -263,7 +283,10 @@ class ContentExporter(UniqueObject, SimpleItem):
                                                               group_id) 
             else:
                 group = portal.portal_groups.getGroupById(group_id)
-                info['principals'] = group.getGroupMembers()
+                try:
+                    info['principals'] = group.getGroupMembers(group_id)
+                except:
+                    info['principals'] = group.getGroupMembers()
                 
             group_info.append(info)
 
@@ -292,12 +315,12 @@ class ContentExporter(UniqueObject, SimpleItem):
                         csv_writer.writerow((id, objtype))
 
         output = ostream.getvalue()
-        # Windows compatibility fix?
-        if os.path.join('a','b')=='a\b':
-            output = output.replace('\r\r\n','\r\n')
+        # Strip Windows line endings 
+        if hasattr(os, 'O_BINARY'):
+            output = output.replace('\r','')
         self.write_file(path,'.objects',output)
 
-    def export(self, portal, root='/'):
+    def export(self, portal, root='/', users='yes', request={}):
         """ Based on generic setup folder export to structre -
             See Products.GenericSetup.interfaces.IFilesystemExporter
         """
@@ -306,6 +329,11 @@ class ContentExporter(UniqueObject, SimpleItem):
         self.lenportal = len(portalname) + 1
         self.out = []
 
+        # Give this a request attribute since some methods expect it
+        # and it is not available when used via current plone 
+        if not hasattr(self, 'REQUEST'):
+            self.REQUEST = request
+
         try:
             if os.path.exists(self.savepath):
                 shutil.rmtree(self.savepath)
@@ -314,18 +342,20 @@ class ContentExporter(UniqueObject, SimpleItem):
             self.out = ['Failed to create structure folder in zope/var',]
             return
 
-        self.export_users(portal)
+        if users:
+            portal.REQUEST = request    
+            self.export_users(portal)
 
         if not root.startswith('/'):
             root = '/%s' % root
         if len(root) > 1 and not root.startswith('/%s/' % portalname):            
             root = '/%s%s' % (portalname, root)            
         catalog = getToolByName(portal, 'portal_catalog')
-        # Enumerate exportable children
-        results = catalog(portal_type=FOLDERS, path={'query': root})
+        # Just return everything then filter for folderish later
+        results = catalog(path={'query': root})
         # exportable = portal.contentItems()
         
-        self.out.append('Exporting %s folders of content to zope/var/structure' % len(results))
+        self.out.append('Exporting %s content items to zope/var/structure' % len(results))
         
         if root == '/':    
             self.write_folder(portal, '/')
@@ -342,9 +372,15 @@ class ContentExporter(UniqueObject, SimpleItem):
                                     )
                     self.export_object(id,rootobj,objtype,'/')
         for brain in results:
-            folder = brain.getObject()
-            path = brain.getPath()
-            self.write_folder(folder, path)
+            try:
+                obj = brain.getObject(self.REQUEST)
+            except:
+                obj = None
+                self.out.append('Object %s at %s was not traversable' % (brain.getId,
+                                                                         brain.getPath()))
+            if obj and obj.isPrincipiaFolderish:
+                path = brain.getPath()
+                self.write_folder(obj, path)
 
         return self.out
 
@@ -368,7 +404,7 @@ class ContentExporter(UniqueObject, SimpleItem):
 
         if hasattr(obj,'data'):
             treatas = 'binary'
-        elif objtype in FOLDERS:
+        elif obj.isPrincipiaFolderish:
             treatas = 'folder'
         else:
             treatas = 'text'
@@ -379,7 +415,9 @@ class ContentExporter(UniqueObject, SimpleItem):
             except:
                 value = getattr(obj,'text', '')
             if text:
-                text += '\n\n%s' % str(value).replace("\r", "")
+                if type(value) not in StringTypes:
+                    value = str(value)
+                text += '\n\n%s' % value.replace("\r", "")
             self.write_file(path,id,text)
         else:
             text = "[DEFAULT]\n%s" % text
@@ -407,7 +445,9 @@ class ContentExporter(UniqueObject, SimpleItem):
             converted to indented strings and clean up line returns in them """
         if isinstance(value, File):
             value = getattr(value, 'data', value)
-        value = str(value).replace("\r", "")
+        if not type(value) in StringTypes:
+            value = str(value)
+        value = value.replace("\r", "")
         if value.endswith("\n"):
             value = value[:-1]
         return value.replace("\n",SEPARATOR)
@@ -443,7 +483,7 @@ class ContentExporter(UniqueObject, SimpleItem):
             #TODO: Add GSXML style ATReference handling all archetypes in plone 2.1 or later        
             if value != None:
                 text += '%s: ' % name
-                if type(value)!=StringType and type(value) in [ListType, TupleType]:
+                if type(value) not in StringTypes and type(value) in [ListType, TupleType]:
                     if value:
                         values = [self.stringify(v) for v in value]
                         text += SEPARATOR.join(values)
