@@ -1,5 +1,5 @@
-FOLDERS = ['Folder','Plone Folder','Large Plone Folder','Plone Memberdata Tool']
 SEPARATOR = "\n  "
+# FOLDERS = ['Folder','Plone Folder','Large Plone Folder','Plone Memberdata Tool']
 # Basic Plone or Zope properties and related methods (ie. doesnt require Archetypes) 
 # NB: Nearly all these mappings actually equate to naming conventions -
 # e.g. drop the underscore and camel case for the accessor but explicit mapping
