@@ -28,7 +28,7 @@ from Products.GenericSetup.utils import _resolveDottedName
 
 from ilrt.contentmigrator.browser.interfaces import IContentMigratorTool
 from ilrt.contentmigrator.exportimport.content import importSiteStructure
-from ilrt.contentmigrator.ContentMigrator.config import FOLDERS, SEPARATOR, PROPS, ALLPROPS
+from ilrt.contentmigrator.ContentMigrator.config import SEPARATOR, PROPS, ALLPROPS
 
 MIGRATOR_TOOL_TYPE = "Content Migrator Tool"
 MIGRATOR_TOOL_ID = "portal_contentsetup"
