@@ -15,7 +15,7 @@ from Products.GenericSetup.content import _globtest
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.exportimport.content import StructureFolderWalkingAdapter
 
-from ilrt.contentmigrator.ContentMigrator.config import FOLDERS, SEPARATOR
+from ilrt.contentmigrator.ContentMigrator.config import SEPARATOR
 from ilrt.contentmigrator.ContentMigrator.config import PROPS
 from ilrt.contentmigrator.browser.contentexporter import ContentExporterView
 from ilrt.contentmigrator.exportimport.utils import atconvert
@@ -110,10 +110,12 @@ class SFWA(StructureFolderWalkingAdapter):
     implements(IFilesystemExporter, IFilesystemImporter)
 
     def __init__(self, context):
+        request = getattr(self,'REQUEST',{})
         self.out = []
         self.context = context
+        self.reindex = request.get('reindex','item')
         if MIGRATE_WORKFLOW:
-            self.wfmt = WorkflowMigrationView(context,getattr(self,'REQUEST',{}))
+            self.wfmt = WorkflowMigrationView(context,request)
             self.transmap = {}
         else:
             self.wfmt = None
@@ -169,18 +171,23 @@ class SFWA(StructureFolderWalkingAdapter):
         delete = our_ids.difference(preserve).union(delete)
 
         for id in prior.intersection(delete):
-            self.out.append('deleting %s' % id)
-            context._delObject(id)
-
+            try:
+                context._delObject(id)
+                self.out.append('Deleted %s' % id)
+            except:
+                self.out.append('Could not delete %s' % id)
+                
         existing = context.objectIds()
         self.out.append('Adding folder %s with %d objects' % (context.getId(),len(ours)))
 
         try:
             for object_id, portal_type in ours:
                 if object_id not in existing:
-                    object = self._makeInstance(object_id, portal_type,
-                                                subdir, import_context)
-
+                    try:
+                        object = self._makeInstance(object_id, portal_type,
+                                                    subdir, import_context)
+                    except:
+                        object = None
                     if object is None:
                         self.out.append("Couldn't make instance: %s/%s" %
                                        (subdir, object_id))
@@ -192,6 +199,9 @@ class SFWA(StructureFolderWalkingAdapter):
                 sfwa = SFWA(wrapped)
                 sfwa.import_(import_context, subdir)
                 self.out.extend(sfwa.out)
+            if getattr(self, 'reindex', 'item') == 'all':
+                catalog = getToolByName(context, 'portal_catalog')
+                catalog.refreshCatalog(clear=1)
         except ValueError:
             err = 'Aborted import due to format error for %s .objects' % subdir
             self.out.append(err)
@@ -210,7 +220,7 @@ class SFWA(StructureFolderWalkingAdapter):
 
         obj = context._getOb(id)
 
-        if portal_type in FOLDERS:
+        if obj.isPrincipiaFolderish:
             meta = import_context.readDataFile('.properties',
                                                  '%s/%s' % (subdir, id))
             if meta:
@@ -244,6 +254,8 @@ class SFWA(StructureFolderWalkingAdapter):
                     else:
                         self.out.append('Found no metadata for obj %s' % obj.getId())
                     self.put_props(obj, meta, import_context, subdir)
+
+        if getattr(self,'reindex','item') == 'item':
             obj.reindexObject()
 
         return obj
@@ -369,8 +381,8 @@ class SFWA(StructureFolderWalkingAdapter):
                         #debug self.out.append(
                         #   'convert %s to %s and add with %s' % (invalue, value, mutator))
                 if not written:
-                    self.out.append('Failed to update %s.%s = %s' % _
-                                            (subdir, obj.getId(), name, value))
+                    self.out.append('Failed to update %s %s.%s = %s' % (subdir,
+                                                     obj.getId(), name, value))
                     
     def properties_demarshall(self, obj, metadata):
         """ Pull out dublin core properties
@@ -380,6 +392,9 @@ class SFWA(StructureFolderWalkingAdapter):
                 if type(value)!=StringType and type(value) in [ListType, TupleType]:
                     value = value.split(SEPARATOR)
                 if obj.hasProperty(key) and value:
-                    obj._updateProperty(key, value)
+                    try:
+                        obj._updateProperty(key, value)
+                    except:
+                        setattr(obj, key, value)
         return
 
